'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var net = require('net');
var contentType = require('content-type');
var typeis = require('type-is');
var fresh = require('fresh');
var accepts = require('accepts');

module.exports = function () {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return function (context, functionDetails, payload) {
    if (payload.serverless) {
      context.request = ServerlessRequest(payload, context.response);
    } else {
      console.warn('The ServerlessRequestProvider should be used with the serverless adapter.');
    }
    return context;
  };
};

function ServerlessRequest(payload, response) {
  var headers = getHeaders(payload);
  var accept = accepts({ headers: headers });
  var request = {
    get header() {
      return headers;
    },
    get headers() {
      return headers;
    },
    get url() {
      return '' + request.origin + request.path;
    },
    get origin() {
      return request.protocol + '://' + request.host;
    },
    get method() {
      return payload.serverless.aws.event.httpMethod;
    },
    get body() {
      if (request.is('json')) {
        try {
          return JSON.parse(request.rawBody);
        } catch (error) {
          console.warn('Unable to parse request body as json: ' + error.message);
        }
      }
      return request.rawBody;
    },
    get rawBody() {
      return payload.serverless.aws.event.body;
    },
    get path() {
      return payload.serverless.aws.event.path;
    },
    get query() {
      return payload.serverless.aws.event.queryStringParameters;
    },
    get params() {
      return payload.serverless.aws.event.pathParameters;
    },
    get host() {
      var host = request.get('x-forwarded-host') || request.get('host');
      if (!host) {
        return '';
      }
      return host.split(/\s*,\s*/)[0];
    },
    get hostname() {
      var host = request.host;
      if (!host) return '';
      return host.split(':')[0];
    },
    get fresh() {
      var method = request.method;
      var s = response.status;

      // GET or HEAD for weak freshness validation only
      if (method !== 'GET' && method !== 'HEAD') return false;

      // 2xx or 304 as per rfc2616 14.26
      if (s >= 200 && s < 300 || s === 304) {
        return fresh(request.header, response.header);
      }

      return false;
    },
    get stale() {
      return !request.fresh;
    },
    get idempotent() {
      var methods = ['GET', 'HEAD', 'PUT', 'DELETE', 'OPTIONS', 'TRACE'];
      return !!~methods.indexOf(request.method);
    },
    get charset() {
      var type = request.get('content-type');
      if (!type) return '';

      return contentType.parse(type).parameters.charset || '';
    },
    get length() {
      var len = request.get('content-length');
      if (len === '') {
        return;
      }
      return ~~len;
    },
    get protocol() {
      var proto = request.get('x-forwarded-proto') || 'http';
      return proto.split(/\s*,\s*/)[0];
    },
    get secure() {
      return request.protocol === 'https';
    },
    get ips() {
      var val = request.get('x-forwarded-for');
      return val ? val.split(/\s*,\s*/) : [];
    },
    get subdomains() {
      var hostname = request.hostname;
      if (net.isIP(hostname)) {
        return [];
      }
      return hostname.split('.').reverse();
    },
    accepts: function accepts() {
      return accept.types.apply(accept, arguments);
    },
    acceptsEncodings: function acceptsEncodings() {
      return accept.encodings.apply(accept, arguments);
    },
    acceptsCharsets: function acceptsCharsets() {
      return accept.charsets.apply(accept, arguments);
    },
    acceptsLanguages: function acceptsLanguages() {
      return accept.languages.apply(accept, arguments);
    },
    is: function is(types) {
      if (!types) {
        return typeis(request);
      }
      if (!Array.isArray(types)) {
        types = [].slice.call(arguments);
      }
      return typeis(request, types);
    },

    get type() {
      var type = request.get('content-type');
      if (!type) {
        return '';
      }
      return type.split(';')[0];
    },
    get: function get(field) {
      var header = request.header;
      switch (field = field.toLowerCase()) {
        case 'referer':
        case 'referrer':
          return header.referrer || header.referer || '';
        default:
          return header[field] || '';
      }
    }
  };

  return request;
}

function getHeaders(payload) {
  var obj = payload.serverless.aws.event.headers;
  var body = payload.serverless.aws.event.body;
  var keys = (0, _keys2.default)(obj);
  var header = {};
  var n = keys.length;
  while (n--) {
    var key = keys[n];
    header[key.toLowerCase()] = obj[key];
  }
  // https://github.com/jshttp/type-is/issues/5
  if (!header['content-length'] && typeof body === 'string') {
    header['content-length'] = body.length;
  }
  return header;
}